from django.contrib.auth.models import User
from django.conf import settings
from rest_framework import authentication

from netaddr import IPAddress


class ApiKeyAuthentication(authentication.BaseAuthentication):
    def authenticate(self, request):
        ALLOWED_SUBNETS = getattr(settings, "API_KEY_ALLOWED_SUBNET", None)
        if not ALLOWED_SUBNETS:
            return None

        client_ip = request.META.get("HTTP_X_FORWARDED_FOR", "").split(",")[0].strip()
        if not client_ip:
            return None

        auto_user = User.objects.get(email=settings.BOT_EMAIL)
        if request.headers.get("x-api-key") == settings.API_KEY:
            for ALLOWED_SUBNET in ALLOWED_SUBNETS:
                if IPAddress(client_ip) in ALLOWED_SUBNET:
                    return auto_user, None

        return None



from django.urls import path, include
from django.contrib.auth.models import User
from rest_framework.decorators import (
    api_view,
    permission_classes,
    authentication_classes,
)
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from django.utils.timezone import now
from django.contrib.sessions.models import Session
from urllib.parse import urljoin
import requests
from django.conf import settings
from django.http import HttpResponseRedirect, HttpResponse, HttpResponseBadRequest


def include_auth_urls():
    return include([path(r"login/", login), path(r"logout/", logout)])


def include_internal_urls():
    return include([path(r"sessions/flush/", flush_session_view),])


@api_view(["GET"])
@permission_classes([AllowAny])
@authentication_classes([])
def login(request):
    auth_redirect_url: str = request.build_absolute_uri().split("?")[0]
    # since the app is running in container, no way to know its on ssl
    auth_redirect_url = auth_redirect_url.replace("http", "https")
    auth_code: str = request.query_params.get("code", "")
    app_redirect_url: str = request.query_params.get("state", "")

    client_id = settings.IDENTITY_CLIENT_ID
    client_secret = settings.IDENTITY_CLIENT_SECRET

    token_url = urljoin(settings.IDENTITY_HOST, "/o/token/")
    reply = requests.post(
        token_url,
        data={
            "grant_type": "authorization_code",
            "code": auth_code,
            "redirect_uri": auth_redirect_url,
        },
        auth=(client_id, client_secret),
    )
    try:
        reply.raise_for_status()
    except requests.exceptions.HTTPError as e:
        message = (
            "Auth request failed with status %s: %s; %s",
            e.response.status_code,
            e.response.content,
            auth_redirect_url,
        )
        return HttpResponse(f"Unauthorized: {message}", status=401)

    tokens = reply.json()
    introspect_url = urljoin(settings.IDENTITY_HOST, "/o/introspect/")
    introspect_response = requests.post(
        introspect_url,
        data={"token": tokens["access_token"],},
        headers={"Authorization": "Bearer " + tokens["access_token"]},
    )
    try:
        reply.raise_for_status()
    except requests.exceptions.HTTPError as e:
        message = (
            "Auth request failed with status %s: %s; %s",
            e.response.status_code,
            e.response.content,
            tokens["access_token"],
        )
        return HttpResponse(f"Unauthorized: {message}", status=401)

    email = introspect_response.json()["username"]
    # here we can also get additional user info, GUID, etc
    user, _ = User.objects.get_or_create(email=email, username=email)
    request.session["username"] = email

    return HttpResponseRedirect(redirect_to=app_redirect_url)


def _get_public_identity_host(request):
    """
    Doing it in a super dumb way for now, lets see if we need smth more complex later
    :return:
    """
    host = request.get_host()
    domain_name = host.split("//")[-1]
    parent_domain = ".".join(domain_name.split(".")[1:])
    return "https://id." + parent_domain


@api_view(["GET"])
@permission_classes([AllowAny])
@authentication_classes([])
def logout(request):
    request.session.flush()
    state: str = request.query_params.get("state", "")
    redirect_url = urljoin(_get_public_identity_host(request), "auth/logout/")
    response = HttpResponseRedirect(redirect_to=f"{redirect_url}?state={state}")
    return response


@api_view(["POST"])
@permission_classes([AllowAny])
@authentication_classes([])
def flush_session_view(request):
    """
    this is supposed to be served under a protected router only accessible from inside the network
    :param request:
    :return:
    """
    email = request.data["email"]
    try:
        user = User.objects.get(email=email)
    except User.DoesNotExist:
        return HttpResponseBadRequest({"reason": "no such user!"})
    delete_all_unexpired_sessions_for_user(user)
    return Response({"flushed_session": f"{email}"})


def delete_all_unexpired_sessions_for_user(user):
    """
    This is pretty inefficient, and will log the user out across all devices
    however, on our scale should work fine.
    We can upgrade this setup later to store session/user pairs in a separate
    custom table
    :param user:
    :return:
    """
    all_sessions = Session.objects.filter(expire_date__gte=now())
    user_sessions = [
        session.pk
        for session in all_sessions
        if str(user.pk) == session.get_decoded().get("_auth_user_id")
    ]
    Session.objects.filter(pk__in=user_sessions).delete()



from urllib.parse import urljoin
from rest_framework import authentication
from django.contrib.auth.models import User
from django.conf import settings
from rest_framework import exceptions
import requests


class IdentityAuthentication(authentication.BaseAuthentication):
    """
    This should be the first authentication that user goess through, since we always trust the local sessions
    So that we dont have to spam api requests to id service
    """

    def authenticate(self, request):
        email = request.session.get("email")
        if not email or email not in settings.IDENTITY_WHITELIST:
            return None
        try:
            user = User.objects.get(email__iexact=email)
        except User.DoesNotExist:
            raise exceptions.AuthenticationFailed("No such user")
        return (user, None)


class IdentitySSOAuthentication(authentication.BaseAuthentication):
    """
    This should be the second authentication that a request goess through:
    When the user doesnt have the local session, we can see if the user has an active session with the id service
    and create local session based on that
    """

    def authenticate(self, request):
        shared_cookie = request.COOKIES.get("sso_shared_session")
        if not shared_cookie:
            # auth not attempted
            return None

        validate_url = urljoin(settings.IDENTITY_HOST, "internal/sessions/validate/")
        user_info_reply = requests.post(
            validate_url, json={"sso_session_cookie": shared_cookie},
        )

        try:
            user_info_reply.raise_for_status()
        except requests.exceptions.HTTPError as e:
            message = (
                "User info request failed",
                e.response.status_code,
                e.response.content,
            )
            raise exceptions.AuthenticationFailed(message)
        user_info = user_info_reply.json()
        email = user_info["user"]["email"]
        user, _ = User.objects.get_or_create(
            email__iexact=email, defaults={"username": email, "email": email}
        )
        request.session["email"] = user.email
        return (user, None)


class IdentityAPIKeyAuthentication(authentication.BaseAuthentication):
    """
    This authentication is used when a user API key is sent
    When the client doesnt have the local session, we try to authenticate it on the ID service
    and create local session based on that
    """

    def authenticate(self, request):
        api_key = request.META.get("HTTP_X_USER_API_KEY")
        if not api_key:
            # auth not attempted
            return None

        validate_url = urljoin(settings.IDENTITY_HOST, "internal/api_keys/validate/")
        user_info_reply = requests.post(validate_url, json={"api_key": api_key},)

        try:
            user_info_reply.raise_for_status()
        except requests.exceptions.HTTPError as e:
            message = (
                "User info request failed",
                e.response.status_code,
                e.response.content,
            )
            raise exceptions.AuthenticationFailed(message)
        user_info = user_info_reply.json()
        email = user_info["user"]["email"]
        user, _ = User.objects.get_or_create(
            email__iexact=email, defaults={"username": email, "email": email}
        )
        request.session["email"] = user.email
        return (user, None)



#!/usr/bin/env python3

from io import open

from setuptools import find_packages, setup


def readall(path):
    with open(path, encoding="utf-8") as fp:
        return fp.read()


setup(
    name="identity_client",
    version="1.0.14",
    description="A middleware and a set of handlers to handle "
    "Code Grant authentication with Cognito",
    long_description=readall("README.md"),
    long_description_content_type="text/markdown",
    author="STITCH",
    author_email="panovitch@gmail.com",
    license="MIT",
    packages=find_packages(exclude=("tests.*", "tests")),
    python_requires=">=3.5",
    install_requires=[
        "Django>=2.2.3,<2.3.0",
        "djangorestframework==3.9.3",
        "requests",
        "netaddr==0.7.19",
    ],
    include_package_data=True,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 2.2",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "License :: OSI Approved :: MIT License",
    ],
)
